#pragma once
#include "object.h"

namespace framework
{
	namespace unity
	{
	#pragma region Իģ
		template<class ValueT, class ConT>
		class IProperty
		{
		protected:
			typedef const ValueT& (ConT::*GetterFunc)() const;
			typedef void (ConT::*SetterFunc)(const ValueT& v);
		};
	#pragma endregion

#pragma region ɶдͨ
		template<class ValueT>
		class Property
		{
		public:
			virtual ~Property(){}
			operator ValueT() const
			{
				return mValue;
			}

			void operator = (const ValueT& value)
			{
				mValue = value;
			}

		protected:
			ValueT mValue;
		};
#pragma endregion

	#pragma region ɶд,ڸֵʱصsetter
		template<class ValueT, class ConT, typename IProperty<ValueT, ConT>::SetterFunc setter>
		class PropertyCallbackSetter : public IProperty<ValueT, ConT>
		{
		public:
			friend ConT;
			typedef IProperty<ValueT, ConT> SuperT;
			typedef PropertyCallbackSetter<ValueT, ConT, setter> ThisT;

			PropertyCallbackSetter(ConT* container)
			{
				mContainer = container;
				mSetter = setter;
			}
			virtual ~PropertyCallbackSetter(){}
			operator ValueT() const
			{
				return mValue;
			}

			void operator = (const ValueT& value)
			{
				(mContainer->*mSetter)(value);
				mValue = value;
			}

		private:
			ConT* mContainer;
			typename SuperT::SetterFunc mSetter;
			ValueT mValue;
		};
	#pragma endregion
	
	#pragma region ֻԣֻͨԱ踺ֵ
		template<class ValueT, class ConT/*, typename IProperty<ValueT, ConT>::SetterFunc setter*/>
		class PropertyReadOnly : public IProperty<ValueT, ConT>
		{
		public:
			friend ConT;
			typedef IProperty<ValueT, ConT> SuperT;
			typedef PropertyReadOnly<ValueT, ConT/*, setter*/> ThisT;
		
			PropertyReadOnly()
			{
				mContainer = NULL;
			}
			PropertyReadOnly(ConT* container)
			{
				mContainer = container;
				//mSetter = setter;
			}
			virtual ~PropertyReadOnly(){}
			operator ValueT() const
			{
				return mValue;
			}

		private:
			void operator = (const ValueT& value)
			{
				//(mContainer->*mSetter)(value);
				mValue = value;
			}

		private:
			ConT* mContainer;
			//typename SuperT::SetterFunc mSetter;
			ValueT mValue;
		};
	#pragma endregion
	
	#pragma region ֻдԣƱֻԸûã
		template<class ValueT, class ConT, typename IProperty<ValueT, ConT>::SetterFunc setter>
		class PropertyWriteOnly : public IProperty<ValueT, ConT>
		{
		public:
			friend ConT;
			typedef IProperty<ValueT, ConT> SuperT;
			typedef PropertyWriteOnly<ValueT, ConT, setter> ThisT;

			PropertyWriteOnly(ConT* container)
			{
				mContainer = container;
				mSetter = setter;
			}
			virtual ~PropertyWriteOnly(){}
			void operator = (const ValueT& value)
			{
				(mContainer->*mSetter)(value);
				mValue = value;
			}
		
		private:
			operator ValueT() const
			{
				return mValue;
			}

		private:
			ConT* mContainer;
			typename SuperT::SetterFunc mSetter;
			ValueT mValue;
		};
	#pragma endregion

#pragma region ֵ,ֻصsettergetter
		template<class ValueT, class ConT, typename IProperty<ValueT, ConT>::SetterFunc setter, typename IProperty<ValueT, ConT>::GetterFunc getter>
		class PropertyNoValue : public IProperty<ValueT, ConT>
		{
		public:
			friend ConT;
			typedef IProperty<ValueT, ConT> SuperT;
			typedef PropertyNoValue<ValueT, ConT, setter, getter> ThisT;

			PropertyNoValue(ConT* container)
			{
				mContainer = container;
				mSetter = setter;
				mGetter = getter;
			}
			virtual ~PropertyNoValue(){}
			operator ValueT() const
			{
				return (mContainer->*mGetter)();
			}

			void operator = (const ValueT& value)
			{
				(mContainer->*mSetter)(value);
			}

		private:
			ConT* mContainer;
			typename SuperT::SetterFunc mSetter;
			typename SuperT::GetterFunc mGetter;
		};
#pragma endregion
	}	// namespace unity
}	// namespace framework

#define PROPERTY_DEFINED_SETTER(name, type, container, setter) public: framework::unity::PropertyCallbackSetter<type, container, &container::setter> name;
#define PROPERTY_READONLY_DEFINED(name, type, container/*, setter*/) public: framework::unity::PropertyReadOnly<type, container/*, &container::setter*/> name;
#define PROPERTY_WRITEONLY_DEFINED(name, type, container, setter) public: framework::unity::PropertyWriteOnly<type, container, &container::setter> name;
#define PROPERTY_DEFINED_NOVALUE(name, type, container, setter, getter) public: framework::unity::PropertyNoValue<type, container, &container::setter, &container::getter> name;

#pragma region Ϊ˽һЩֳɵ鷽ΪԣĿǰִֻ췽
#define PROPERTY_DEFINED_NOVALUE_VIRTUAL(name, type, container, super, setter, getter) \
	protected: void setter(const type& v){ super::setter(v); } \
	protected: const type& getter() const{ return super::getter(); } \
	public: framework::unity::PropertyNoValue<type, container, &container::setter, &container::getter> name;
#pragma endregion